import { Injectable, Logger } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { CODE } from 'src/common/code';
import { GoodsStatus, UserOnlieStatus } from 'src/common/const';
import { ExternalApiService } from 'src/common/utils/external-api';
import { CustomResult, SuccessResult } from 'src/common/utils/result-utils';
import { GoodsService } from 'src/goods/goods.service';
import { CreateOrderlogDto } from 'src/orderlog/dto/create-orderlog.dto';
import { OrderlogService } from 'src/orderlog/orderlog.service';
import { Repository } from 'typeorm';
import { BuyGoodsUserDto } from './dto/buygoods-user.dto';
import { CreateUserDto } from './dto/create-user.dto';
import { UpdateUserDto } from './dto/update-user.dto';
import { UserEntity } from './entities/user.entity';
import { UserModel, AppUserModel } from './user.model';

const prefix = 'user';
const logger = new Logger('UserService');
@Injectable()
export class UserService {
  constructor(
    @InjectRepository(UserEntity)
    private readonly userRepository: Repository<UserEntity>,
    private readonly goodsService: GoodsService,
    private readonly orderLogService: OrderlogService
  ) { }

  /**查询用户信息,没有则初始化 */
  async init(createUserDto: CreateUserDto) {
    //查询用户信息
    const userInfo = await this.findOne(createUserDto.uid);
    //有信息则返回
    const appUserInfo = await ExternalApiService.getAppUserInfo(createUserDto.uid, createUserDto.token);
    if (userInfo) return userInfo;
    //用户信息异常处理
    if (!appUserInfo || appUserInfo.code != 200) return CustomResult(CODE.CODE_USER_INFO_ERR);
    //没有信息则初始化
    const initRes = await this.initUser(appUserInfo);
    if (!initRes) return CustomResult(CODE.CODE_INIT_USER_ERR);

    return initRes;
  }
  /**用户信息初始化 */
  private async initUser(appUser: AppUserModel) {
    const entity: UserModel = {
      userid: appUser.id,
      avatar: appUser.avatar,
      nickname: appUser.nickname,
      fodder_balance: 0,
      eggs: 0,
      online_status: UserOnlieStatus.ONLINE,
      parent_id: 0,
      create_time: new Date(),
      update_time: new Date()
    }
    return await this.userRepository.save(entity);
  }

  /**查询用户信息 */
  async findOne(userid: number) {
    const info = await this.userRepository.createQueryBuilder(prefix)
      .select()
      .where('userid = :userid', { userid: userid })
      .getRawOne();
    return info;
  }

  /**更新用户数据 */
  async update(userid: number, updateUserDto: UpdateUserDto) {
    updateUserDto.update_time = new Date();
    const res = await this.userRepository.createQueryBuilder()
      .update()
      .set(updateUserDto)
      .where('userid = :userid', { userid: userid })
      .execute();
    if (!res) return CustomResult(CODE.CODE_USER_UPDATE_ERR)
    return SuccessResult();
  }

  /** 用户购买商品逻辑 */
  async buy(buyGoodsUserDto: BuyGoodsUserDto) {
    //查询APP用户信息
    const appUserInfo = await ExternalApiService.getAppUserInfo(buyGoodsUserDto.userid, buyGoodsUserDto.token);
    if (appUserInfo?.code != 200) return appUserInfo;
    const user = appUserInfo.data;

    //查询商品信息
    const goodsInfo: any = await this.goodsService.findOne(buyGoodsUserDto.goodsid, GoodsStatus.HIDE);
    if (goodsInfo?.code != 200) return CustomResult(CODE.CODE_GOODS_INFO_ERR);
    const goods = goodsInfo.data;

    //判断用户金额是否满足条件 (商品价格*商品数量)
    const payCoin = goods.goods_price * buyGoodsUserDto.goods_count;//应支付价格
    if (user.coin < payCoin) return CustomResult(CODE.CODE_APPUSER_COIN_NOT_ENOUGHT_ERR);

    //TODO 扣减加锁 锁几秒

    //用户金额扣减
    const subUserInfo = await ExternalApiService.subAppUserCoin(buyGoodsUserDto.userid, buyGoodsUserDto.token, payCoin);
    console.log('扣减后用户信息',subUserInfo);

    //发放完成记录日志
    const orderInfo: CreateOrderlogDto = {
      userid: buyGoodsUserDto.userid,
      goods_id: goods.goods_id,
      goods_type: goods.goods_type,
      goods_price: goods.goods_price,
      goods_attribute: goods.goods_attribute
    }
    // const orderRes = await this.orderLogService.create(orderInfo);
    // logger.debug(`购买订单日志结果: ${JSON.stringify(orderRes)}`);
    // if (orderRes.code != 200) return CustomResult(CODE.CODE_GOODS_DEAL_ERR);

    return appUserInfo;
    return SuccessResult();
  }


}
